from typing import (
    Generic,
    List,
    Tuple,
    Sequence,
    Iterator,
    Optional,
    Union,
    overload,
)
from positionlib.position import Position
from ..common import Number, T

class Vector(Generic[T]):
    def __init__(self, data: Union[Sequence[T], "Position[T]"]) -> None: ...
    def __setattr__(self, name: str, value: object) -> None: ...
    @property
    def ndim(self) -> int: ...
    def to_list(self) -> List[T]: ...
    def to_tuple(self) -> Tuple[T, ...]: ...
    def norm(self) -> float: ...
    def normalize(self) -> "Vector[float]": ...
    def dot(self, other: "Vector[T]") -> float: ...
    def __add__(self, other: "Vector[T]") -> "Vector[float]": ...
    def __sub__(self, other: "Vector[T]") -> "Vector[float]": ...
    def __repr__(self) -> str: ...
    def __getitem__(self, idx: int) -> T: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[T]: ...
    def __eq__(self, other: object) -> bool: ...
    def __mul__(self, scalar: Number) -> "Vector[float]": ...
    def __rmul__(self, scalar: Number) -> "Vector[float]": ...
    def distance(self, other: "Vector[T]") -> float: ...
    def manhattan(self, other: "Vector[T]") -> T: ...
    def lerp(self, other: "Vector[T]", t: float) -> "Vector[float]": ...
    def clamp(self, min_val: Number, max_val: Number) -> "Vector[float]": ...
    def abs(self) -> "Vector[float]": ...
    def sum(self) -> T: ...
    def prod(self) -> T: ...
    def min(self) -> T: ...
    def max(self) -> T: ...
    def is_unit(self, tol: float = ...) -> bool: ...
    def inverse(self) -> "Vector[float]": ...
    def astype(self, dtype: type) -> "Vector[float]": ...
    def reflect(self, normal: "Vector[T]") -> "Vector[float]": ...
    def project(self, other: "Vector[T]") -> "Vector[float]": ...
    def angle_between(self, other: "Vector[T]") -> float: ...
    def _get_coord(self, index: int) -> T: ...
    def _cast_val(self, value: float) -> T: ...
    def _cast_coords(self, coords) -> List[T]: ...
    def _create(self, data: Sequence[Number]) -> "Vector[float]": ...
    def _from_arr(self, arr) -> "Vector[float]": ...
    def _inv_coords(self) -> List[float]: ...
    def _refl_coords(self, normal: "Vector[T]") -> List[float]: ...
    def _proj_coords(self, other: "Vector[T]") -> List[float]: ...

class Vec2(Vector[T]):
    @overload
    def __init__(self, x: Position[T], y: None = ...) -> None: ...
    @overload
    def __init__(self, x: T, y: T) -> None: ...
    def __init__(self, x: Union[T, Position[T]], y: Optional[T] = None): ...
    def _create(self, data: Sequence[Number]) -> "Vec2[float]": ...
    @property
    def x(self) -> T: ...
    @property
    def y(self) -> T: ...
    def cross(self, other: "Vec2[T]") -> T: ...
    def angle(self, other: "Vec2[T]") -> float: ...
    def inverse(self) -> "Vec2[float]": ...
    def reflect(self, normal: "Vec2[T]") -> "Vec2[float]": ...  # type: ignore[override]
    def project(self, other: "Vec2[T]") -> "Vec2[float]": ...  # type: ignore[override]

class Vec3(Vector[T]):
    @overload
    def __init__(self, x: Position[T], y: None = None, z: None = None) -> None: ...
    @overload
    def __init__(self, x: T, y: T, z: T) -> None: ...
    def __init__(
        self, x: Union[T, Position[T]], y: Optional[T] = None, z: Optional[T] = None
    ): ...
    def _create(self, data: Sequence[Number]) -> "Vec3[float]": ...
    @property
    def x(self) -> T: ...
    @property
    def y(self) -> T: ...
    @property
    def z(self) -> T: ...
    def cross(self, other: "Vec3[T]") -> "Vec3[float]": ...
    def angle(self, other: "Vec3[T]") -> float: ...
    def inverse(self) -> "Vec3[float]": ...
    def reflect(self, normal: "Vec3[T]") -> "Vec3[float]": ...  # type: ignore[override]
    def project(self, other: "Vec3[T]") -> "Vec3[float]": ...  # type: ignore[override]

class Vec4(Vector[T]):
    @overload
    def __init__(
        self, x: Position[T], y: None = None, z: None = None, w: None = None
    ) -> None: ...
    @overload
    def __init__(self, x: T, y: T, z: T, w: T) -> None: ...
    def __init__(
        self,
        x: Union[T, Position[T]],
        y: Optional[T] = None,
        z: Optional[T] = None,
        w: Optional[T] = None,
    ): ...
    def _create(self, data: Sequence[Number]) -> "Vec4[float]": ...
    @property
    def x(self) -> T: ...
    @property
    def y(self) -> T: ...
    @property
    def z(self) -> T: ...
    @property
    def w(self) -> T: ...
    def inverse(self) -> "Vec4[float]": ...
    def reflect(self, normal: "Vec4[T]") -> "Vec4[float]": ...  # type: ignore[override]
    def project(self, other: "Vec4[T]") -> "Vec4[float]": ...  # type: ignore[override]
